try:
    from builtins import str, range
except ImportError:
    pass

from unittest import TestCase
from flask_blogging import BloggingEngine, PostProcessor
from markdown.extensions.codehilite import CodeHiliteExtension


sample_markdown = """

##This is a test


    :::python
    print("Hello, World")
"""

expected_markup = u'<h2>This is a test</h2>\n' \
                  u'<div class="codehilite"><pre>' \
                  u'<span class="k">print</span>' \
                  u'<span class="p">(</span><span class="s">' \
                  u'&quot;Hello, World&quot;</span><span class="p">)' \
                  u'</span>\n' \
                  u'</pre></div>'


class TestCore(TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_custom_md_extension(self):
        extn = CodeHiliteExtension({})
        engine = BloggingEngine(extensions=[extn])
        extns = engine.post_processor.all_extensions()
        self.assertEqual(len(extns), 3)
        self.assertTrue(isinstance(extns[-1], CodeHiliteExtension))



from flask import Flask, render_template_string, redirect, current_app
from sqlalchemy import create_engine, MetaData
from flask_login import UserMixin, LoginManager, login_user, logout_user, current_user
from flask_blogging import SQLAStorage, BloggingEngine
from flask_principal import identity_changed, Identity, AnonymousIdentity, identity_loaded, \
    UserNeed, RoleNeed


app = Flask(__name__)
app.config["SECRET_KEY"] = "secret"  # for WTF-forms and login
app.config["BLOGGING_URL_PREFIX"] = "/blog"
app.config["BLOGGING_DISQUS_SITENAME"] = "test"
app.config["BLOGGING_SITEURL"] = "http://localhost:8000"
app.config["BLOGGING_SITENAME"] = "My Site"
app.config["BLOGGING_PERMISSIONS"] = True  # Enable blogger permissions

# extensions
engine = create_engine('sqlite:////tmp/blog.db')
meta = MetaData()
sql_storage = SQLAStorage(engine, metadata=meta)
blog_engine = BloggingEngine(app, sql_storage)
login_manager = LoginManager(app)
meta.create_all(bind=engine)

class User(UserMixin):
    def __init__(self, user_id):
        self.id = user_id

    def get_name(self):
        return "Paul Dirac"  # typically the user's name


@identity_loaded.connect_via(app)
def on_identity_loaded(sender, identity):
    identity.user = current_user
    if hasattr(current_user, "id"):
        identity.provides.add(UserNeed(current_user.id))
    #identity.provides.add(RoleNeed("blogger"))


@login_manager.user_loader
@blog_engine.user_loader
def load_user(user_id):
    return User(user_id)

index_template = """
<!DOCTYPE html>
<html>
    <head> </head>
    <body>
        {% if current_user.is_authenticated %}
            <a href="/logout/"> Logout </a>
        {% else %}
            <a href="/login/"> Login </a>
        {% endif %}
        &nbsp&nbsp<a href="/blog/"> Blog </a>
        &nbsp&nbsp<a href="/blog/sitemap.xml">Sitemap</a>
        &nbsp&nbsp<a href="/blog/feeds/all.atom.xml">ATOM</a>
    </body>
</html>
"""

@app.route("/")
def index():
    return render_template_string(index_template)

@app.route("/login/")
def login():
    user = User("testuser")
    login_user(user)

    # notify the change of role
    identity_changed.send(current_app._get_current_object(),
                          identity=Identity("testuser"))
    return redirect("/blog")

@app.route("/logout/")
def logout():
    logout_user()

    # notify the change of role
    identity_changed.send(current_app._get_current_object(),
                          identity=AnonymousIdentity())
    return redirect("/")


if __name__ == "__main__":
    app.run(debug=True, port=8000, use_reloader=True)



from flask_blogging import signals
from flask_blogging.sqlastorage import SQLAStorage
import sqlalchemy as sqla
from sqlalchemy import func


def get_tag_data(sqla_storage):
    engine = sqla_storage.engine
    with engine.begin() as conn:
        tag_posts_table = sqla_storage.tag_posts_table
        tag_table = sqla_storage.tag_table

        tag_cloud_stmt = sqla.select([
            tag_table.c.text,func.count(tag_posts_table.c.tag_id)]).group_by(
            tag_posts_table.c.tag_id
        ).where(tag_table.c.id == tag_posts_table.c.tag_id).limit(10)
        tag_cloud = conn.execute(tag_cloud_stmt).fetchall()
    return tag_cloud


def get_tag_cloud(app, engine, posts, meta, count, page):
    if isinstance(engine.storage, SQLAStorage):
        tag_cloud = get_tag_data(engine.storage)
        meta["tag_cloud"] = tag_cloud
    else:
        raise RuntimeError("Plugin only supports SQLAStorage. Given storage"
                           "not supported")
    return


def register(app):
    signals.index_posts_fetched.connect(get_tag_cloud)
    return



from flask_blogging import signals, BloggingEngine
from flask import Blueprint
from werkzeug.contrib.atom import AtomFeed


# receivers for various signals
def blueprint_created_receiver(sender, engine, blueprint):
    assert sender == engine.app
    isinstance(engine, BloggingEngine)
    isinstance(blueprint, Blueprint)
    engine.ctr_blueprint_created += 1


def sitemap_posts_receiver(sender, engine, posts):
    assert sender == engine.app
    isinstance(engine, BloggingEngine)
    isinstance(posts, list)
    engine.ctr_sitemap_posts += 1


def feed_posts_fetched_receiver(sender, engine, posts):
    assert sender == engine.app
    isinstance(engine, BloggingEngine)
    isinstance(posts, list)
    engine.ctr_feed_posts_fetched += 1


def feed_posts_processed_receiver(sender, engine, feed):
    assert sender == engine.app
    isinstance(engine, BloggingEngine)
    isinstance(feed, AtomFeed)
    engine.ctr_feed_posts_processed += 1


def index_posts_receiver(sender, engine, posts, meta, count, page):
    assert sender == engine.app
    isinstance(engine, BloggingEngine)
    isinstance(posts, list)
    isinstance(meta, dict)
    isinstance(count, int)
    isinstance(page, int)
    engine.ctr_index_posts += 1


def page_by_id_receiver(sender, engine, post, meta, post_id, slug):
    assert sender == engine.app
    isinstance(engine, BloggingEngine)
    isinstance(post, dict)
    isinstance(meta, dict)
    isinstance(post_id, int)
    isinstance(slug, str)
    engine.ctr_page_by_id += 1


def posts_by_tag_receiver(sender, engine, posts, meta, tag, count, page):
    assert sender == engine.app
    isinstance(engine, BloggingEngine)
    isinstance(posts, list)
    isinstance(meta, dict)
    isinstance(tag, str)
    isinstance(count, int)
    isinstance(page, int)
    engine.ctr_posts_by_tag += 1


def posts_by_author_receiver(sender, engine, posts, meta, user_id, count,
                             page):
    assert sender == engine.app
    isinstance(engine, BloggingEngine)
    isinstance(posts, list)
    isinstance(meta, dict)
    isinstance(user_id, str)
    isinstance(count, int)
    isinstance(page, int)
    engine.ctr_posts_by_author += 1


def register(app):
    signals.blueprint_created.connect(blueprint_created_receiver)

    signals.sitemap_posts_fetched.connect(sitemap_posts_receiver)
    signals.sitemap_posts_processed.connect(sitemap_posts_receiver)

    signals.feed_posts_fetched.connect(feed_posts_fetched_receiver)
    signals.feed_posts_processed.connect(feed_posts_processed_receiver)

    signals.index_posts_fetched.connect(index_posts_receiver)
    signals.index_posts_processed.connect(index_posts_receiver)

    signals.page_by_id_fetched.connect(page_by_id_receiver)
    signals.page_by_id_processed.connect(page_by_id_receiver)

    signals.posts_by_tag_fetched.connect(posts_by_tag_receiver)
    signals.posts_by_tag_processed.connect(posts_by_tag_receiver)

    signals.posts_by_author_fetched.connect(posts_by_author_receiver)
    signals.posts_by_author_processed.connect(posts_by_author_receiver)


def disconnect_receivers(app):
    signals.blueprint_created.disconnect(blueprint_created_receiver)

    signals.sitemap_posts_fetched.disconnect(sitemap_posts_receiver)
    signals.sitemap_posts_processed.disconnect(sitemap_posts_receiver)

    signals.feed_posts_fetched.disconnect(feed_posts_fetched_receiver)
    signals.feed_posts_processed.disconnect(feed_posts_processed_receiver)

    signals.index_posts_fetched.disconnect(index_posts_receiver)
    signals.index_posts_processed.disconnect(index_posts_receiver)

    signals.page_by_id_fetched.disconnect(page_by_id_receiver)
    signals.page_by_id_processed.disconnect(page_by_id_receiver)

    signals.posts_by_tag_fetched.disconnect(posts_by_tag_receiver)
    signals.posts_by_tag_processed.disconnect(posts_by_tag_receiver)

    signals.posts_by_author_fetched.disconnect(posts_by_author_receiver)
    signals.posts_by_author_processed.disconnect(posts_by_author_receiver)



def ensureUtf(s, encoding='utf8'):
    """Converts input to unicode if necessary.
    If `s` is bytes, it will be decoded using the `encoding` parameters.
    This function is used for preprocessing /source/ and /filename/ arguments
    to the builtin function `compile`.
    """
    # In Python2, str == bytes.
    # In Python3, bytes remains unchanged, but str means unicode
    # while unicode is not defined anymore
    if type(s) == bytes:
        return s.decode(encoding, 'ignore')
    else:
        return s



"""
Flask-Blogging
--------------

Flask-Blogging is a Flask extension for adding blogging support to
your web application. Flask-Blogging comes with the following
features out of the box:

- Bootstrap based site
- Markdown based blog editor
- Models to store blog
- Authentication of User's choice
- Sitemap, ATOM support
- Disqus support for comments
- Google analytics for usage tracking
- Permissions enabled to control which users can create/edit blogs
- Incorporates Flask-Cache for fast rendering
- Well documented, tested, and extensible design


Links
`````
* `documentation <http://flask-blogging.readthedocs.org/>`_
* `development version <https://github.com/gouthambs/Flask-Blogging>`_

"""

from setuptools import setup, find_packages
import os
import re
import ast

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('flask_blogging/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))



BASE_PATH = os.path.dirname(__file__)


def get_requirements(suffix=''):
    with open(os.path.join(BASE_PATH, 'Requirements%s.txt' % suffix)) as f:
        rv = f.read().splitlines()
    return rv

setup(
    name='Flask-Blogging',
    version=version,
    url='https://github.com/gouthambs/Flask-Blogging',
    license='MIT',
    author='Gouthaman Balaraman',
    author_email='gouthaman.balaraman@gmail.com',
    description='A flask extension for adding Markdown blog support to your site',
    long_description=__doc__,
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=get_requirements(),
    tests_require=["nose", "mysqlclient", "psycopg2"],
    test_suite='nose.collector',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



from flask_wtf import Form
from wtforms import StringField, TextAreaField, SubmitField, BooleanField
from wtforms.validators import DataRequired


class BlogEditor(Form):
    title = StringField("title", validators=[DataRequired()])
    text = TextAreaField("text", validators=[DataRequired()])
    tags = StringField("tags", validators=[DataRequired()])
    draft = BooleanField("draft", default=False)
    submit = SubmitField("submit")
